@@ -2,7 +2,7 @@
 
 #include <libical/ical.h>
 
-void program_errors()
+void program_errors(void)
 {
     /*Most routines will set icalerrno on errors. This is an
       enumeration defined in icalerror.h */
@@ -43,7 +43,7 @@ void component_errors(icalcomponent *comp)
 
 
 
-    /* Check the component for iTIP compilance, and add more
+    /* Check the component for iTIP complance, and add more
        X-LIC-ERROR properties if it is non-compilant. */
     icalrestriction_check(comp);
 
