@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 #Exit if any undefined variable is used.
 set -u
@@ -39,6 +39,7 @@ HELP() {
   echo " -g, --no-gcc-build     Don't run any gcc-build tests"
   echo " -a, --no-asan-build    Don't run any ASAN-build tests"
   echo " -d, --no-tsan-build    Don't run any TSAN-build tests"
+  echo " -f, --no-fortify-build Don't run the FORTIFY-build tests (gcc12)"
   echo
 }
 
@@ -46,7 +47,11 @@ COMMAND_EXISTS () {
     command -v $1 >/dev/null 2>&1
     if ( test $? != 0 )
     then
-    echo "$1 is not in your PATH. Either install this program or skip the assocatied test"
+    echo "$1 is not in your PATH. Either install this program or skip the associated test"
+    if ( test "$2" )
+    then
+      echo "or disable this check by passing the $2 command-line option"
+    fi
     exit 1
   fi
 }
@@ -109,7 +114,7 @@ COMPILE_WARNINGS() {
 # print warnings found in the cppcheck output
 # $1 = file with the cppcheck output
 CPPCHECK_WARNINGS() {
-  CHECK_WARNINGS $1 "\(warning\|error\|information\|portability\)" ""
+  CHECK_WARNINGS "$1" "\(warning\|error\|portability\)" ""
 }
 
 #function TIDY_WARNINGS:
@@ -117,16 +122,16 @@ CPPCHECK_WARNINGS() {
 # $1 = file with the clang-tidy output
 TIDY_WARNINGS() {
   #whitelist='\(Value[[:space:]]descriptions\|unused[[:space:]]declarations\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|modernize-\|cppcoreguidelines-pro-type-const-cast\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-type-reinterpret-cast\|cppcoreguidelines-owning-memory\|fuchsia.*\|hicpp-use-auto\|hicpp-no-malloc\|hicpp-use-nullptr\|hicpp-exception-baseclass\|hicpp-vararg\|cppcoreguidelines-pro-type-vararg\|cppcoreguidelines-pro-bounds-pointer-arithmetic\|google-build-using-namespace\|llvm-include-order\|hicpp-use-equals-default\|cppcoreguidelines-no-malloc\|g_type_class_add_private.*is[[:space:]]deprecated\)'
-  whitelist='\(no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\)'
-  CHECK_WARNINGS $1 "warning:" "$whitelist"
+  whitelist='\(no[[:space:]]link[[:space:]]for:\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|G_ADD_PRIVATE\|g_type_class_add_private.*is[[:space:]]deprecated\|g-ir-scanner:\|clang.*argument[[:space:]]unused[[:space:]]during[[:space:]]compilation\|libical-glib-scan\.c\)'
+  CHECK_WARNINGS "$1" "warning:" "$whitelist"
 }
 
 #function SCAN_WARNINGS:
 # print warnings found in the scan-build output
 # $1 = file with the scan-build output
 SCAN_WARNINGS() {
-  whitelist='\(no[[:space:]]link[[:space:]]for:\|g_type_class_add_private.*is[[:space:]]deprecated\|libical-glib-scan\.c\|/i-cal-object\.c\|/vcc\.c\|/vobject\.c\|/icalsslexer\.c\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|icalerror.*Dereference[[:space:]]of[[:space:]]null[[:space:]]pointer\|G_ADD_PRIVATE\)'
-  CHECK_WARNINGS $1 "warning:" $whitelist
+  whitelist='\(no[[:space:]]link[[:space:]]for:\|g_type_class_add_private.*is[[:space:]]deprecated\|libical-glib-scan\.c\|/i-cal-object\.c\|/vcc\.c\|/vobject\.c\|/icalsslexer\.c\|Value[[:space:]]descriptions\|unused[[:space:]]declarations\|icalerror.*Dereference[[:space:]]of[[:space:]]null[[:space:]]pointer\|G_ADD_PRIVATE\|g-ir-scanner:\)'
+  CHECK_WARNINGS "$1" "warning:" "$whitelist"
 }
 
 #function CONFIGURE:
@@ -138,7 +143,12 @@ CONFIGURE() {
   mkdir -p $BDIR
   cd $BDIR
   rm -rf *
-  cmake .. $2 || exit 1
+  if ( test `echo $2 | grep -ci Ninja` -gt 0 )
+  then
+    cmake .. $2 || exit 1
+  else
+    cmake -G "Unix Makefiles" .. $2 || exit 1
+  fi
 }
 
 #function CLEAN:
@@ -191,6 +201,31 @@ GCC_BUILD() {
   echo "===== END GCC BUILD: $1 ======"
 }
 
+#function FORTIFY_BUILD:
+# runs a build test using gcc (v12 or higher) with fortify CFLAGS
+# $1 = the name of the test (which will have "-fortify" appended to it)
+# $2 = CMake options
+FORTIFY_BUILD() {
+  name="$1-fortify"
+  if ( test $runfortifybuild -ne 1 )
+  then
+    echo "===== FORTIFY BUILD TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
+    return
+  fi
+  COMMAND_EXISTS "gcc"
+  gccVersion=`gcc -dumpversion`
+  if ( test `expr $gccVersion + 0` -lt 12 )
+  then
+    echo "Sorry, gcc must be version 12 or higher to support fortify. Exiting..."
+    exit 1
+  fi
+  echo "===== START FORTIFY BUILD: $1 ======"
+  SET_GCC
+  export CFLAGS="-Og -gdwarf-5 -fno-optimize-sibling-calls -Wall -W -U_FORTIFY_SOURCE -D_FORTIFY_SOURCE=3 -D_FORTIFY_SOURCE=3 -fPIC -grecord-gcc-switches -fno-allow-store-data-races -fstack-protector-strong -fstack-clash-protection -fcf-protection=full --param=ssp-buffer-size=1"
+  BUILD "$name" "$2"
+  echo "===== END FORTIFY BUILD: $1 ======"
+}
+
 #function NINJA_GCC_BUILD:
 # runs a build test using gcc using the Ninja cmake generator
 # $1 = the name of the test (which will have "-ninjagcc" appended to it)
@@ -228,7 +263,7 @@ CLANG_BUILD() {
 }
 
 #function ASAN_BUILD:
-# runs an clang ASAN build test
+# runs a clang ASAN build test
 # $1 = the name of the test (which will have "-asan" appended to it)
 # $2 = CMake options
 ASAN_BUILD() {
@@ -239,7 +274,9 @@ ASAN_BUILD() {
     return
   fi
   echo "===== START ASAN BUILD: $1 ======"
-  SET_CLANG
+  SET_GCC # I'm using ld.gold (vs ld.bfd), which doesn't play well with clang and asan=>use gcc
+  #SET_CLANG currently has linking problems using ld.gold
+  export ASAN_OPTIONS=verify_asan_link_order=0 #seems to be needed with different ld on Fedora (like gold)
   BUILD "$name" "-DADDRESS_SANITIZER=True $2"
   echo "===== END ASAN BUILD: $1 ======"
 }
@@ -272,47 +309,45 @@ CPPCHECK() {
     echo "===== CPPCHECK TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "cppcheck"
+  COMMAND_EXISTS "cppcheck" "-c"
   echo "===== START SETUP FOR CPPCHECK: $1 ======"
 
   #first build it
-  cd $TOP
+  cd "$TOP" || exit 1
   SET_GCC
   CONFIGURE "$name" "$2"
   make 2>&1 | tee make.out || exit 1
 
   echo "===== START CPPCHECK: $1 ======"
-  cd $TOP
-  cppcheck --quiet --language=c \
-           --force --error-exitcode=1 --inline-suppr \
-           --enable=warning,performance,portability,information \
-           --template='{file}:{line},{severity},{id},{message}' \
-           -D sleep="" \
-           -D localtime_r="" \
-           -D gmtime_r="" \
-           -D size_t="unsigned long" \
-           -D bswap32="" \
-           -D PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP=0 \
-           -D _unused="(void)" \
-           -D F_OK=0 \
-           -D R_OK=0 \
-           -U YYSTYPE \
-           -U PVL_USE_MACROS \
-           -I $BDIR \
-           -I $BDIR/src/libical \
-           -I $BDIR/src/libicalss \
-           -I $TOP/src/libical \
-           -I $TOP/src/libicalss \
-           -I $TOP/src/libicalvcal \
-           $TOP/src $BDIR/src/libical/icalderived* 2>&1 | \
-      grep -v 'Found a statement that begins with numeric constant' | \
-      grep -v 'cannot find all the include files' | \
-      grep -v Net-ICal | \
-      grep -v icalssyacc\.c  | \
-      grep -v icalsslexer\.c | \
-      grep -v _cxx\. | tee cppcheck.out
+  cd "$TOP" || exit 1
+  cppcheck --quiet \
+    --language=c++ \
+    --std=c++11 \
+    --force --error-exitcode=1 --inline-suppr \
+    --enable=warning,performance,portability \
+    --check-level=exhaustive \
+    --disable=missingInclude \
+    --suppress=passedByValue --suppress=ctuOneDefinitionRuleViolation \
+    --template='{file}:{line},{severity},{id},{message}' \
+    --checkers-report=cppcheck-report.txt \
+    -U PVL_USE_MACROS \
+    -I "$BDIR" \
+    -I "$BDIR/src/libical" \
+    -I "$BDIR/src/libicalss" \
+    -I "$BDIR/src/libicalvcal" \
+    -I "$BDIR/src/libical-glib" \
+    -I "$TOP/src/libical" \
+    -I "$TOP/src/libicalss" \
+    -I "$TOP/src/libicalvcal" \
+    -I "$TOP/src/libical-glib" \
+    -i "$TOP/src/Net-ICal-Libical" \
+    -i "$TOP/src/libicalss/icalssyacc.c" \
+    -i "$TOP/src/libicalss/icalsslexer.c" \
+    -i "$TOP/src/libicalvcal/vcc.c" \
+    "$TOP/src" "$BDIR"/src/libical/icalderived* 2>&1 |
+    tee cppcheck.out
   CPPCHECK_WARNINGS cppcheck.out
-  rm -f cppcheck.out
+  rm -f cppcheck.out cppcheck-report.txt
   CLEAN
   echo "===== END CPPCHECK: $1 ======"
 }
@@ -328,7 +363,7 @@ SPLINT() {
     echo "===== SPLINT TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "splint"
+  COMMAND_EXISTS "splint" "-s"
   echo "===== START SETUP FOR SPLINT: $1 ======"
 
   #first build it
@@ -351,6 +386,7 @@ SPLINT() {
   files="$files $BDIR/src/libical/*.c $BDIR/src/libical/*.h"
 
   splint $files \
+       -badflag \
        -weak -warnposix \
        -modobserver -initallelements -redef \
        -linelen 1000 \
@@ -412,7 +448,7 @@ CLANGTIDY() {
     echo "===== CLANG-TIDY TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "clang-tidy"
+  COMMAND_EXISTS "clang-tidy" "-t"
   echo "===== START CLANG-TIDY: $1 ====="
   cd $TOP
   SET_CLANG
@@ -433,16 +469,17 @@ CLANGSCAN() {
     echo "===== SCAN-BUILD TEST $1 DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "scan-build"
+  COMMAND_EXISTS "scan-build" "-b"
   echo "===== START SCAN-BUILD: $1 ====="
-  cd $TOP
+  cd "$TOP" || exit 1
 
   #configure specially with scan-build
   SET_BDIR "$1-scan"
-  mkdir -p $BDIR
-  cd $BDIR
-  rm -rf *
-  scan-build cmake .. "$2" || exit 1
+  rm -rf "$BDIR"
+  mkdir "$BDIR"
+  cd "$BDIR" || exit 1
+  # shellcheck disable=SC2086
+  scan-build cmake .. $2 || exit 1
 
   scan-build make 2>&1 | tee make-scan.out || exit 1
   SCAN_WARNINGS make-scan.out
@@ -458,7 +495,7 @@ KRAZY() {
     echo "===== KRAZY TEST DISABLED DUE TO COMMAND LINE OPTION ====="
     return
   fi
-  COMMAND_EXISTS "krazy2all"
+  COMMAND_EXISTS "krazy2all" "-k"
   echo "===== START KRAZY ====="
   cd $TOP
   krazy2all 2>&1 | tee krazy.out
@@ -474,8 +511,8 @@ KRAZY() {
 
 ##### END FUNCTIONS #####
 
-#TEMP=`getopt -o hmkctbsnlgad --long help,no-cmake-compat,no-krazy,no-cppcheck,no-tidy,no-scan,no-splint,no-ninja,no-clang-build,no-gcc-build,no-asan-build,no-tsan-build -- "$@"`
-TEMP=`getopt hmkctbsnlgad $*`
+#TEMP=`getopt -o hmkctbsnlgadf --long help,no-cmake-compat,no-krazy,no-cppcheck,no-tidy,no-scan,no-splint,no-ninja,no-clang-build,no-gcc-build,no-asan-build,no-tsan-build,no-fortify-build -- "$@"`
+TEMP=`getopt hmkctbsnlgadf $*`
 if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi
 # Note the quotes around `$TEMP': they are essential!
 eval set -- "$TEMP"
@@ -490,21 +527,23 @@ runclangbuild=1
 rungccbuild=1
 runasanbuild=1
 runtsanbuild=1
+runfortifybuild=1
 runsplint=1
 while true ; do
     case "$1" in
         -h|--help) HELP; exit 1;;
-        -m|--no-cmake-compat) cmakecompat=0;   shift;;
-        -k|--no-krazy)        runkrazy=0;      shift;;
-        -c|--no-cppcheck)     runcppcheck=0;   shift;;
-        -t|--no-tidy)         runtidy=0;       shift;;
-        -b|--no-scan)         runscan=0;       shift;;
-        -s|--no-splint)       runsplint=0;     shift;;
-        -n|--no-ninja)        runninja=0;      shift;;
-        -l|--no-clang-build)  runclangbuild=0; shift;;
-        -g|--no-gcc-build)    rungccbuild=0;   shift;;
-        -a|--no-asan-build)   runasanbuild=0;  shift;;
-        -d|--no-tsan-build)   runtsanbuild=0;  shift;;
+        -m|--no-cmake-compat)  cmakecompat=0;     shift;;
+        -k|--no-krazy)         runkrazy=0;        shift;;
+        -c|--no-cppcheck)      runcppcheck=0;     shift;;
+        -t|--no-tidy)          runtidy=0;         shift;;
+        -b|--no-scan)          runscan=0;         shift;;
+        -s|--no-splint)        runsplint=0;       shift;;
+        -n|--no-ninja)         runninja=0;        shift;;
+        -l|--no-clang-build)   runclangbuild=0;   shift;;
+        -g|--no-gcc-build)     rungccbuild=0;     shift;;
+        -a|--no-asan-build)    runasanbuild=0;    shift;;
+        -d|--no-tsan-build)    runtsanbuild=0;    shift;;
+        -f|--no-fortify-build) runfortifybuild=0; shift;;
         --) shift; break;;
         *)  echo "Internal error!"; exit 1;;
     esac
@@ -526,7 +565,7 @@ then
     exit 1
   fi
   # read the min required CMake version from the top-level CMake file
-  minCMakeVers=`grep -i cmake_minimum_required $TOP/CMakeLists.txt | grep VERSION | sed 's/^.*VERSION\s*//' | cut -d. -f1-2 | sed 's/\s*).*$//'`
+  minCMakeVers=`grep -i cmake_minimum_required $TOP/CMakeLists.txt | grep VERSION | sed 's/^.*VERSION\s*//' | cut -d. -f1-2 | sed 's/\s*).*$//' | awk '{print $NF}'`
   # adjust PATH
   X=`echo $minCMakeVers | cut -d. -f1`
   Y=`echo $minCMakeVers | cut -d. -f2`
@@ -550,27 +589,38 @@ then
   fi
 fi
 
+DEFCMAKEOPTS="-DCMAKE_BUILD_TYPE=Release -DICAL_GLIB=False -DNDEBUG=1"
 CMAKEOPTS="-DCMAKE_BUILD_TYPE=Debug -DGOBJECT_INTROSPECTION=False -DICAL_GLIB=False -DICAL_BUILD_DOCS=False"
 UUCCMAKEOPTS="$CMAKEOPTS -DCMAKE_DISABLE_FIND_PACKAGE_ICU=True"
 TZCMAKEOPTS="$CMAKEOPTS -DUSE_BUILTIN_TZDATA=True"
-GLIBOPTS="-DCMAKE_BUILD_TYPE=Debug -DGOBJECT_INTROSPECTION=True -DUSE_BUILTIN_TZDATA=OFF -DICAL_GLIB_VAPI=ON"
+GLIBOPTS="-DCMAKE_BUILD_TYPE=Debug -DICAL_GLIB=True -DGOBJECT_INTROSPECTION=True -DUSE_BUILTIN_TZDATA=OFF -DICAL_GLIB_VAPI=OFF"
+FUZZOPTS="-DLIBICAL_BUILD_TESTING_BIGFUZZ=True"
 
 #Static code checkers
+#TODO: Building with vapi gives a bus error (see issue #771)
+STATICCCHECKOPTS="\
+-DCMAKE_BUILD_TYPE=Debug \
+-DSHARED_ONLY=True \
+-DWITH_CXX_BINDINGS=True \
+-DUSE_BUILTIN_TZDATA=True \
+-DICAL_GLIB=True \
+-DGOBJECT_INTROSPECTION=True \
+-DICAL_GLIB_VAPI=False \
+-DENABLE_GTK_DOC=True \
+-DLIBICAL_BUILD_TESTING=True \
+-DLIBICAL_BUILD_EXAMPLES=True \
+"
 KRAZY
-SPLINT test2 "$CMAKEOPTS"
-SPLINT test2builtin "$TZCMAKEOPTS"
-CPPCHECK test2 "$CMAKEOPTS"
-CPPCHECK test2builtin "$TZCMAKEOPTS"
-CLANGSCAN test2 "$CMAKEOPTS"
-CLANGSCAN test2builtin "$TZCMAKEOPTS"
-CLANGTIDY test2 "$CMAKEOPTS"
-CLANGTIDY test2builtin "$TZCMAKEOPTS"
+SPLINT test "$STATICCCHECKOPTS"
+CLANGSCAN test "$STATICCCHECKOPTS"
+CLANGTIDY test "$STATICCCHECKOPTS"
+CPPCHECK test "$STATICCCHECKOPTS"
 
 #GCC based build tests
-GCC_BUILD testgcc1 ""
+GCC_BUILD testgcc1 "$DEFCMAKEOPTS"
 GCC_BUILD testgcc2 "$CMAKEOPTS"
 GCC_BUILD testgcc3 "$UUCCMAKEOPTS"
-GCC_BUILD testgcc4glib "$GLIBOPTS"
+#GCC_BUILD testgcc4glib "$GLIBOPTS"
 if (test "`uname -s`" = "Linux")
 then
     echo "Temporarily disable cross-compile tests"
@@ -581,14 +631,19 @@ GCC_BUILD testgcc1builtin "-DUSE_BUILTIN_TZDATA=True"
 GCC_BUILD testgcc2builtin "$TZCMAKEOPTS"
 
 #Ninja build tests
-NINJA_GCC_BUILD testninjagcc1 ""
-NINJA_GCC_BUILD testninjagcc2 "-DSHARED_ONLY=True"
-NINJA_GCC_BUILD testninjagcc3 "-DSTATIC_ONLY=True -DICAL_GLIB=False"
+NINJA_GCC_BUILD testninjagcc1 "$DEFCMAKEOPTS"
+NINJA_GCC_BUILD testninjagcc2 "-DICAL_GLIB=True"
+NINJA_GCC_BUILD testninjagcc3 "-DICAL_GLIB=True -DICAL_GLIB_VAPI=ON -DGOBJECT_INTROSPECTION=True"
+NINJA_GCC_BUILD testninjagcc4 "-DSHARED_ONLY=True -DICAL_GLIB=False"
+NINJA_GCC_BUILD testninjagcc5 "-DSHARED_ONLY=True -DICAL_GLIB=True"
+NINJA_GCC_BUILD testninjagcc6 "-DSTATIC_ONLY=True -DICAL_GLIB=False"
+NINJA_GCC_BUILD testninjagcc7 "-DSTATIC_ONLY=True -DICAL_GLIB=True -DENABLE_GTK_DOC=False"
+NINJA_GCC_BUILD testninjagcc9 "-DSHARED_ONLY=True -DICAL_GLIB=True -DGOBJECT_INTROSPECTION=True -DICAL_GLIB_VAPI=ON"
 
-CLANG_BUILD testclang1 ""
+CLANG_BUILD testclang1 "$DEFCMAKEOPTS"
 CLANG_BUILD testclang2 "$CMAKEOPTS"
 CLANG_BUILD testclang3 "$UUCCMAKEOPTS"
-#broken with clang7 on Fedora29 CLANG_BUILD testclang4glib "$GLIBOPTS"
+CLANG_BUILD testclang4glib "$GLIBOPTS"
 if (test "`uname -s`" = "Linux")
 then
     echo "Temporarily disable cross-compile tests"
@@ -597,17 +652,27 @@ then
 fi
 
 #Address sanitizer
-ASAN_BUILD test1asan ""
+ASAN_BUILD test1asan "$DEFCMAKEOPTS"
 ASAN_BUILD test2asan "$CMAKEOPTS"
 ASAN_BUILD test3asan "$TZCMAKEOPTS"
 ASAN_BUILD test4asan "$UUCCMAKEOPTS"
 ASAN_BUILD test5asan "$GLIBOPTS"
+ASAN_BUILD test6asan "$FUZZOPTS"
 
 #Thread sanitizer
-TSAN_BUILD test1tsan ""
+TSAN_BUILD test1tsan "$DEFCMAKEOPTS"
 TSAN_BUILD test2tsan "$CMAKEOPTS"
 TSAN_BUILD test3tsan "$TZCMAKEOPTS"
 TSAN_BUILD test4tsan "$UUCCMAKEOPTS"
 TSAN_BUILD test5tsan "$GLIBOPTS"
+TSAN_BUILD test6tsan "$FUZZOPTS"
+
+#Fortify build
+FORTIFY_BUILD test1fortify "$DEFCMAKEOPTS"
+FORTIFY_BUILD test2tsan "$CMAKEOPTS"
+FORTIFY_BUILD test3tsan "$TZCMAKEOPTS"
+FORTIFY_BUILD test4tsan "$UUCCMAKEOPTS"
+#FORTIFY_BUILD test5tsan "$GLIBOPTS" #bus error in libical-glib.vapi
+FORTIFY_BUILD test6fortify "$FUZZOPTS"
 
 echo "ALL TESTS COMPLETED SUCCESSFULLY"
