@@ -113,10 +113,10 @@ icalcomponent *icalcomponent_new(icalcomponent_kind kind)
     return icalcomponent_new_impl(kind);
 }
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wvarargs"
 icalcomponent *icalcomponent_vanew(icalcomponent_kind kind, ...)
 {
+    /* See https://github.com/libical/libical/issues/585. Caller must pass NULL as final argument */
+
     va_list args;
 
     icalcomponent *impl = icalcomponent_new_impl(kind);
@@ -131,7 +131,6 @@ icalcomponent *icalcomponent_vanew(icalcomponent_kind kind, ...)
 
     return impl;
 }
-#pragma clang diagnostic pop
 
 icalcomponent *icalcomponent_new_from_string(const char *str)
 {
@@ -785,6 +784,21 @@ static int icalcomponent_is_busy(icalcomponent *comp)
     return (ret);
 }
 
+static struct icaltimetype icaltime_with_time(const struct icaltimetype t, int hour, int minutes, int seconds)
+{
+    struct icaltimetype ret = t;
+    ret.hour = hour;
+    ret.minute = minutes;
+    ret.second = seconds;
+    ret.is_date = 0;
+    return ret;
+}
+
+static struct icaltimetype icaltime_at_midnight(const struct icaltimetype t)
+{
+    return icaltime_with_time(t, 0, 0, 0);
+}
+
 void icalcomponent_foreach_recurrence(icalcomponent *comp,
                                       struct icaltimetype start,
                                       struct icaltimetype end,
@@ -822,10 +836,22 @@ void icalcomponent_foreach_recurrence(icalcomponent *comp,
 
     basespan.is_busy = icalcomponent_is_busy(comp);
 
+    if (start.is_date) {
+        /* We always treat start as date-time, because we do arithmetic calculations later
+           on that wouldn't work on date-only. As date-only values shouldn't have a timezone set,
+           we shouldn't have any issues with potential DST changes. */
+        start = icaltime_at_midnight(start);
+    }
+
     /* Calculate the ceiling and floor values.. */
     limit_start = icaltime_as_timet_with_zone(start,
                                               icaltimezone_get_utc_timezone());
     if (!icaltime_is_null_time(end)) {
+        if (end.is_date) {
+            /* Same as with start, treat as date-time to allow for arithmetic operations. */
+            end = icaltime_at_midnight(end);
+        }
+
         limit_end = icaltime_as_timet_with_zone(end,
                                                 icaltimezone_get_utc_timezone());
     } else {
@@ -865,7 +891,8 @@ void icalcomponent_foreach_recurrence(icalcomponent *comp,
             icaltimetype mystart = start;
 
             /* make sure we include any recurrence that ends in timespan */
-            icaltime_adjust(&mystart, 0, 0, 0, -(int)dtduration);
+            /* we ensured above that start is a date-time, so adding seconds is allowed. */
+            icaltime_adjust(&mystart, 0, 0, 0, -(int)(long)dtduration);
             icalrecur_iterator_set_start(rrule_itr, mystart);
         }
 
@@ -906,27 +933,40 @@ void icalcomponent_foreach_recurrence(icalcomponent *comp,
 
         struct icaldatetimeperiodtype rdate_period =
             icalproperty_get_rdate(rdate);
+        struct icaltimetype rdate_start = rdate_period.time;
+        time_t rdate_duration = 0;
 
         /* RDATES can specify raw datetimes, periods, or dates.
-            we only support raw datetimes for now..
+            we only support raw datetimes and periods for now.
 
             @todo Add support for other types */
 
-        if (icaltime_is_null_time(rdate_period.time))
-            continue;
+        if (icaltime_is_null_time(rdate_start)) {
+            rdate_start = rdate_period.period.start;
+
+            if (icaltime_is_null_time(rdate_period.period.end)) {
+                rdate_duration =
+                    (time_t)icaldurationtype_as_int(rdate_period.period.duration);
+            } else {
+                recurspan.end =
+                    icaltime_as_timet_with_zone(rdate_period.period.end,
+                                                rdate_period.time.zone ? rdate_period.time.zone : icaltimezone_get_utc_timezone());
+            }
+        } else {
+            rdate_duration = dtduration;
+        }
 
         recurspan.start =
-            icaltime_as_timet_with_zone(rdate_period.time,
-                                        rdate_period.time.zone ?
-                                        rdate_period.time.zone :
-                                        icaltimezone_get_utc_timezone());
-        recurspan.end = recurspan.start + dtduration;
+            icaltime_as_timet_with_zone(rdate_start,
+                                        rdate_period.time.zone ? rdate_period.time.zone : icaltimezone_get_utc_timezone());
+        if (rdate_duration)
+            recurspan.end = recurspan.start + rdate_duration;
 
         /* save the iterator ICK! */
         property_iterator = comp->property_iterator;
 
         if (!icalproperty_recurrence_is_excluded(comp,
-                                                 &dtstart, &rdate_period.time)) {
+                                                 &dtstart, &rdate_start)) {
             /* call callback action */
             if (icaltime_span_overlaps(&recurspan, &limit_span))
                 (*callback) (comp, &recurspan, callback_data);
@@ -2427,6 +2467,15 @@ static int prop_compare(void *a, void *b)
     return r;
 }
 
+static inline int compare_nullptr(const void *a, const void *b)
+{
+    if (!a == !b)
+        return 0;
+
+    // non-NULL sorts before NULL
+    return a ? -1 : 1;
+}
+
 static int comp_compare(void *a, void *b)
 {
     icalcomponent *c1 = (icalcomponent*) a;
@@ -2436,7 +2485,7 @@ static int comp_compare(void *a, void *b)
     int r = k1 - k2;
 
     if (r == 0) {
-        if (k1 == ICAL_X_COMPONENT) {
+        if (k1 == ICAL_X_COMPONENT && (c1->x_name && c2->x_name)) {
             r = strcmp(c1->x_name, c2->x_name);
         }
 
@@ -2460,17 +2509,25 @@ static int comp_compare(void *a, void *b)
                                                           ICAL_TRIGGER_PROPERTY);
                     p2 = icalcomponent_get_first_property(c2,
                                                           ICAL_TRIGGER_PROPERTY);
-                    r = strcmp(icalproperty_get_value_as_string(p1),
-                               icalproperty_get_value_as_string(p2));
-
-                    if (r == 0) {
-                        p1 = icalcomponent_get_first_property(c1,
-                                                              ICAL_ACTION_PROPERTY);
-                        p2 = icalcomponent_get_first_property(c2,
-                                                              ICAL_ACTION_PROPERTY);
+                    if (p1 && p2) {
                         r = strcmp(icalproperty_get_value_as_string(p1),
                                    icalproperty_get_value_as_string(p2));
+                        if (r == 0) {
+                            p1 = icalcomponent_get_first_property(c1,
+                                                                  ICAL_ACTION_PROPERTY);
+                            p2 = icalcomponent_get_first_property(c2,
+                                                                  ICAL_ACTION_PROPERTY);
+                            if (p1 && p2) {
+                                r = strcmp(icalproperty_get_value_as_string(p1),
+                                           icalproperty_get_value_as_string(p2));
+                            } else {
+                                r = compare_nullptr(p1, p2);
+                            }
+                        }
+                    } else {
+                        r = compare_nullptr(p1, p2);
                     }
+
                     break;
 
                 case ICAL_VTIMEZONE_COMPONENT:
@@ -2478,8 +2535,12 @@ static int comp_compare(void *a, void *b)
                                                           ICAL_TZID_PROPERTY);
                     p2 = icalcomponent_get_first_property(c2,
                                                           ICAL_TZID_PROPERTY);
-                    r = strcmp(icalproperty_get_value_as_string(p1),
-                               icalproperty_get_value_as_string(p2));
+                    if (p1 && p2) {
+                        r = strcmp(icalproperty_get_value_as_string(p1),
+                                   icalproperty_get_value_as_string(p2));
+                    } else {
+                        r = compare_nullptr(p1, p2);
+                    }
                     break;
 
                 case ICAL_XSTANDARD_COMPONENT:
@@ -2488,8 +2549,13 @@ static int comp_compare(void *a, void *b)
                                                           ICAL_DTSTART_PROPERTY);
                     p2 = icalcomponent_get_first_property(c2,
                                                           ICAL_DTSTART_PROPERTY);
-                    r = strcmp(icalproperty_get_value_as_string(p1),
-                               icalproperty_get_value_as_string(p2));
+
+                    if (p1 && p2) {
+                        r = strcmp(icalproperty_get_value_as_string(p1),
+                                   icalproperty_get_value_as_string(p2));
+                    } else {
+                        r = compare_nullptr(p1, p2);
+                    }
                     break;
 
                 case ICAL_VVOTER_COMPONENT:
@@ -2497,8 +2563,13 @@ static int comp_compare(void *a, void *b)
                                                           ICAL_VOTER_PROPERTY);
                     p2 = icalcomponent_get_first_property(c2,
                                                           ICAL_VOTER_PROPERTY);
-                    r = strcmp(icalproperty_get_value_as_string(p1),
-                               icalproperty_get_value_as_string(p2));
+
+                    if (p1 && p2) {
+                        r = strcmp(icalproperty_get_value_as_string(p1),
+                                   icalproperty_get_value_as_string(p2));
+                    } else {
+                        r = compare_nullptr(p1, p2);
+                    }
                     break;
 
                 case ICAL_XVOTE_COMPONENT:
@@ -2506,8 +2577,13 @@ static int comp_compare(void *a, void *b)
                                                           ICAL_POLLITEMID_PROPERTY);
                     p2 = icalcomponent_get_first_property(c2,
                                                           ICAL_POLLITEMID_PROPERTY);
-                    r = strcmp(icalproperty_get_value_as_string(p1),
-                               icalproperty_get_value_as_string(p2));
+
+                    if (p1 && p2) {
+                        r = strcmp(icalproperty_get_value_as_string(p1),
+                                   icalproperty_get_value_as_string(p2));
+                    } else {
+                        r = compare_nullptr(p1, p2);
+                    }
                     break;
 
                 default:
@@ -2530,10 +2606,17 @@ static int comp_compare(void *a, void *b)
 
 void icalcomponent_normalize(icalcomponent *comp)
 {
-    pvl_list sorted_props = pvl_newlist();
-    pvl_list sorted_comps = pvl_newlist();
     icalproperty *prop;
     icalcomponent *sub;
+    pvl_list sorted_props;
+    pvl_list sorted_comps;
+
+    icalerror_check_arg(comp != 0, "comp");
+    if (!comp)
+        return;
+
+    sorted_props = pvl_newlist();
+    sorted_comps = pvl_newlist();
 
     /* Normalize properties into sorted list */
     while ((prop = pvl_pop(comp->properties)) != 0) {
