@@ -2,30 +2,29 @@
  FILE: icalparser.c
  CREATOR: eric 04 August 1999
 
- (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
-     http://www.softwarestudio.org
+ (C) COPYRIGHT 2000, Eric Busboom <eric@civicknowledge.com>
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
-    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
 
  Or:
 
     The Mozilla Public License Version 2.0. You may obtain a copy of
-    the License at http://www.mozilla.org/MPL/
+    the License at https://www.mozilla.org/MPL/
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
-    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
 
  Or:
 
     The Mozilla Public License Version 2.0. You may obtain a copy of
-    the License at http://www.mozilla.org/MPL/
+    the License at https://www.mozilla.org/MPL/
 
   The Initial Developer of the Original Code is Eric Busboom
  ======================================================================*/
@@ -48,6 +47,8 @@
 #define MAXIMUM_ALLOWED_PARAMETERS 100
 #define MAXIMUM_ALLOWED_MULTIPLE_VALUES 500
 
+static enum icalparser_ctrl icalparser_ctrl_g = ICALPARSER_CTRL_KEEP;
+
 struct icalparser_impl
 {
     int buffer_full;    /* flag indicates that temp is smaller that
@@ -170,9 +171,14 @@ static char *parser_get_next_char(char c, char *str, int qm)
     return 0;
 }
 
-/** make a new tmp buffer out of a substring */
+/** Makes a new tmp buffer out of a substring. */
 static char *make_segment(char *start, char *end)
 {
+#if defined(__GNUC__) && !defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#pragma GCC diagnostic ignored "-Wstringop-overflow"
+#endif
     char *buf, *tmp;
     ptrdiff_t size = (ptrdiff_t)(end - start);
 
@@ -187,6 +193,9 @@ static char *make_segment(char *start, char *end)
     }
 
     return buf;
+#if defined(__GNUC__) && !defined(__clang__)
+#pragma GCC diagnostic pop
+#endif
 }
 
 static char *parser_get_prop_name(char *line, char **end)
@@ -214,7 +223,7 @@ static char *parser_get_prop_name(char *line, char **end)
     return str;
 }
 
-/* Decode parameter value per RFC6868 */
+/** Decode parameter value per RFC6868 */
 static void parser_decode_param_value(char *value)
 {
     char *in, *out;
@@ -264,7 +273,7 @@ static int parser_get_param_name_stack(char *line, char *name, size_t name_lengt
         /* Dequote the value */
         next++;
 
-        end_quote = parser_get_next_char('"', next, 0);
+        end_quote = (*next == '"') ? next : parser_get_next_char('"', next, 0);
 
         if (end_quote == 0) {
             return 0;
@@ -311,7 +320,7 @@ static char *parser_get_param_name_heap(char *line, char **end)
     *end = next + 1;
     if (**end == '"') {
         *end = *end + 1;
-        next = parser_get_next_char('"', *end, 0);
+        next = (**end == '"') ? *end : parser_get_next_char('"', *end, 0);
         if (next == 0) {
             free(str);
             *end = NULL;
@@ -393,7 +402,7 @@ static char *parser_get_next_value(char *line, char **end, icalvalue_kind kind)
 
         next = parser_get_next_char(',', p, 1);
 
-        /* Unforunately, RFC2445 allowed that for the RECUR value, COMMA
+        /* Unfortunately, RFC2445 allowed that for the RECUR value, COMMA
            could both separate digits in a list, and it could separate
            multiple recurrence specifications. This is not a friendly
            part of the spec and was deprecated in RFC5545. The following
@@ -410,8 +419,8 @@ static char *parser_get_next_value(char *line, char **end, icalvalue_kind kind)
                 continue;
             }
         }
-        /* ignore all , for query value. select dtstart, dtend etc ... */
-        else if (kind == ICAL_QUERY_VALUE) {
+        /* ignore all commas for query and x values. select dtstart, dtend etc ... */
+        else if (kind == ICAL_QUERY_VALUE || kind == ICAL_X_VALUE) {
             if (next != 0) {
                 p = next + 1;
                 continue;
@@ -474,10 +483,6 @@ static char *parser_get_next_parameter(char *line, char **end)
     }
 }
 
-/**
- * Get a single property line, from the property name through the
- * final new line, and include any continuation lines
- */
 char *icalparser_get_line(icalparser *parser,
                           icalparser_line_gen_func line_gen_func)
 {
@@ -546,7 +551,7 @@ char *icalparser_get_line(icalparser *parser,
             if (parser->temp[0] == '\0') {
 
                 if (line[0] != '\0') {
-                    /* There is data in the output, so fall trhough and process it */
+                    /* There is data in the output, so fall through and process it */
                     break;
                 } else {
                     /* No data in output; return and signal that there
@@ -609,7 +614,7 @@ static void insert_error(icalcomponent *comp, const char *text,
 
     icalcomponent_add_property(
         comp,
-        icalproperty_vanew_xlicerror(temp, icalparameter_new_xlicerrortype(type), 0));
+        icalproperty_vanew_xlicerror(temp, icalparameter_new_xlicerrortype(type), (void *)0));
 }
 
 static int line_is_blank(char *line)
@@ -709,6 +714,47 @@ icalcomponent *icalparser_add_line(icalparser *parser, char *line)
         return 0;
     }
 
+    if (icalparser_ctrl_g != ICALPARSER_CTRL_KEEP) {
+        static const unsigned char is_icalctrl[256] = {
+            1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1,
+            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+        };
+
+        char *c, *d;
+        for (c = d = line; *c; c++) {
+            if (!is_icalctrl[(unsigned char)*c]) {
+                *d++ = *c;
+            } else if (icalparser_ctrl_g == ICALPARSER_CTRL_OMIT) {
+                // omit CTRL character
+            } else {
+                icalcomponent *tail = pvl_data(pvl_tail(parser->components));
+                if (tail) {
+                    insert_error(
+                            tail, line,
+                            "Content line contains invalid CONTROL characters",
+                            ICAL_XLICERRORTYPE_COMPONENTPARSEERROR);
+                }
+                parser->state = ICALPARSER_ERROR;
+                return 0;
+            }
+        }
+        *d = '\0';
+    }
+
     /* Begin by getting the property name at the start of the line. The
        property name may end up being "BEGIN" or "END" in which case it
        is not really a property, but the marker for the start or end of
@@ -788,8 +834,8 @@ icalcomponent *icalparser_add_line(icalparser *parser, char *line)
 
         if (parser->level < 0) {
             // Encountered an END before any BEGIN, this must be invalid data
-            icalerror_warn("Encountered END before BEGIN");
-
+            icalerror_set_error_state(ICAL_MALFORMEDDATA_ERROR,
+                                      icalerror_get_error_state(ICAL_MALFORMEDDATA_ERROR));
             parser->state = ICALPARSER_ERROR;
             parser->level = 0;
             return 0;
@@ -906,7 +952,7 @@ icalcomponent *icalparser_add_line(icalparser *parser, char *line)
 
                 if (name_heap == 0) {
                     /* 'tail' defined above */
-                    insert_error(tail, str, "Cant parse parameter name",
+                    insert_error(tail, str, "Can't parse parameter name",
                                  ICAL_XLICERRORTYPE_PARAMETERNAMEPARSEERROR);
                     tail = 0;
                     break;
@@ -1017,7 +1063,7 @@ icalcomponent *icalparser_add_line(icalparser *parser, char *line)
                 /* Change for mozilla */
                 /* have the option of being flexible towards unsupported parameters */
 #if ICAL_ERRORS_ARE_FATAL == 1
-                insert_error(tail, str, "Cant parse parameter name",
+                insert_error(tail, str, "Can't parse parameter name",
                              ICAL_XLICERRORTYPE_PARAMETERNAMEPARSEERROR);
                 tail = 0;
                 parser->state = ICALPARSER_ERROR;
@@ -1059,7 +1105,7 @@ icalcomponent *icalparser_add_line(icalparser *parser, char *line)
 
             if (param == 0) {
                 /* 'tail' defined above */
-                insert_error(tail, str, "Cant parse parameter value",
+                insert_error(tail, str, "Can't parse parameter value",
                              ICAL_XLICERRORTYPE_PARAMETERVALUEPARSEERROR);
 
                 tail = 0;
@@ -1098,6 +1144,7 @@ icalcomponent *icalparser_add_line(icalparser *parser, char *line)
 
                     icalmemory_free_buffer(str);
                     str = NULL;
+                    pcount++;
                     continue;
                 }
             }
@@ -1324,7 +1371,15 @@ char *icalparser_string_line_generator(char *out, size_t buf_size, void *d)
         size = buf_size - 1;
     }
 
+#if defined(__GNUC__) && !defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#pragma GCC diagnostic ignored "-Wstringop-overflow"
+#endif
     strncpy(out, data->pos, size);
+#if defined(__GNUC__) && !defined(__clang__)
+#pragma GCC diagnostic pop
+#endif
 
     if(replace_cr) {
         *(out + size - 1) = '\n';
@@ -1360,3 +1415,13 @@ icalcomponent *icalparser_parse_string(const char *str)
 
     return c;
 }
+
+enum icalparser_ctrl icalparser_get_ctrl(void)
+{
+    return icalparser_ctrl_g;
+}
+
+void icalparser_set_ctrl(enum icalparser_ctrl ctrl)
+{
+    icalparser_ctrl_g = ctrl;
+}
