@@ -2,8 +2,7 @@
  FILE: icaltime.c
  CREATOR: eric 02 June 2000
 
- (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
-     http://www.softwarestudio.org
+ (C) COPYRIGHT 2000, Eric Busboom <eric@civicknowledge.com>
 
  The timegm code is Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
 
@@ -11,12 +10,12 @@
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
-    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
 
  Or:
 
     The Mozilla Public License Version 2.0. You may obtain a copy of
-    the License at http://www.mozilla.org/MPL/
+    the License at https://www.mozilla.org/MPL/
 
  The Original Code is eric. The Initial Developer of the Original
  Code is Eric Busboom
@@ -76,66 +75,49 @@ static int icaltime_leap_days(int y1, int y2)
 }
 
 /*
- * Code adapted from Python 2.4.1 sources (Lib/calendar.py).
- */
-static time_t icaltime_timegm(const struct tm *tm)
-{
-    int year;
-    time_t days;
-    time_t hours;
-    time_t minutes;
-    time_t seconds;
-
-    year = 1900 + tm->tm_year;
-    days = (time_t)(365 * (year - 1970) + icaltime_leap_days(1970, year));
-    days += days_in_year_passed_month[0][tm->tm_mon];
-
-    if (tm->tm_mon > 1 && icaltime_is_leap_year(year))
-        ++days;
-
-    days += tm->tm_mday - 1;
-    hours = days * 24 + tm->tm_hour;
-    minutes = hours * 60 + tm->tm_min;
-    seconds = minutes * 60 + tm->tm_sec;
-
-    return seconds;
-}
-
-/*
- *  Function to convert a struct tm time specification
  *  to an ANSI time_t using the specified time zone.
  *  This is different from the standard mktime() function
  *  in that we don't want the automatic adjustments for
  *  local daylight savings time applied to the result.
  *  This function expects well-formed input.
+ *
+ *  The out_time_t is to store the result, it can be NULL.
+ *  Returns 0 on failure, 1 on success.
  */
-static time_t make_time(struct tm *tm, int tzm)
+static int make_time(const struct tm *tm, int tzm, time_t *out_time_t)
 {
     time_t tim;
+    int febs;
 
     static int days[] = { -1, 30, 58, 89, 119, 150, 180, 211, 242, 272, 303, 333, 364 };
 
     /* check that month specification within range */
 
     if (tm->tm_mon < 0 || tm->tm_mon > 11)
-        return ((time_t) - 1);
+        return 0;
+
+    if (tm->tm_year < 2)
+        return 0;
 
 #if (SIZEOF_TIME_T == 4)
     /* check that year specification within range */
 
-    if (tm->tm_year < 70 || tm->tm_year > 138)
-        return ((time_t) - 1);
-
-    /* check for upper bound of Jan 17, 2038 (to avoid possibility of
-       32-bit arithmetic overflow) */
+    if (tm->tm_year > 138)
+        return 0;
 
+    /* check for upper bound of Jan 17, 2038 (to avoid possibility of 32-bit arithmetic overflow) */
     if (tm->tm_year == 138) {
         if (tm->tm_mon > 0) {
-            return ((time_t) - 1);
+            return 0;
         } else if (tm->tm_mday > 17) {
-            return ((time_t) - 1);
+            return 0;
         }
     }
+#else
+    /* We don't support years >= 10000, because the function has not been tested at this range. */
+    if (tm->tm_year >= 8100) {
+        return 0;
+    }
 #endif /* SIZEOF_TIME_T */
 
     /*
@@ -146,12 +128,17 @@ static time_t make_time(struct tm *tm, int tzm)
 
     tim = (time_t) ((tm->tm_year - 70) * 365 + ((tm->tm_year - 1) / 4) - 17);
 
+    /* adjust: no leap days every 100 years, except every 400 years. */
+
+    febs = (tm->tm_year - 100) - ((tm->tm_mon <= 1) ? 1 : 0);
+    tim -= febs / 100;
+    tim += febs / 400;
+
     /* add number of days elapsed in the current year */
 
     tim += days[tm->tm_mon];
 
-    /* check and adjust for leap years (the leap year check only valid
-       during the 32-bit era */
+    /* check and adjust for leap years */
 
     if ((tm->tm_year & 3) == 0 && tm->tm_mon > 1)
         tim += 1;
@@ -178,33 +165,44 @@ static time_t make_time(struct tm *tm, int tzm)
 
     /* return number of seconds since start of the epoch */
 
-    return (tim);
+    if (out_time_t)
+        *out_time_t = tim;
+
+    return 1;
 }
 
-/**     @brief Constructor.
- *
- *      @param tm The time
- *      @param is_date Boolean: 1 means we should treat tm as a DATE
- *      @param zone The timezone tm is in, NULL means to treat tm as a
- *              floating time
- *
- *      Return a new icaltime instance, initialized to the given time
- *      expressed as seconds past UNIX epoch, optionally using the given
- *      timezone.
- *
- *      If the caller specifies the is_date param as TRUE, the returned
- *      object is of DATE type, otherwise the input is meant to be of
- *      DATE-TIME type.
- *      If the zone is not specified (NULL zone param) the time is taken
- *      to be floating, that is, valid in any timezone. Note that, in
- *      addition to the uses specified in [RFC5545], this can be used
- *      when doing simple math on couples of times.
- *      If the zone is specified (UTC or otherwise), it's stored in the
- *      object and it's used as the native timezone for this object.
- *      This means that the caller can convert this time to a different
- *      target timezone with no need to store the source timezone.
- *
+/*
+ * Code adapted from Python 2.4.1 sources (Lib/calendar.py).
  */
+static time_t icaltime_timegm(const struct tm *tm)
+{
+    int year;
+    time_t days;
+    time_t hours;
+    time_t minutes;
+    time_t seconds;
+
+    /* Validate the tm structure by passing it through make_time() */
+    if (!make_time(tm, 0, NULL)) {
+        /* we have some invalid data in the tm struct */
+        return 0;
+    }
+
+    year = 1900 + tm->tm_year;
+    days = (time_t)(365 * (year - 1970) + icaltime_leap_days(1970, year));
+    days += days_in_year_passed_month[0][tm->tm_mon];
+
+    if (tm->tm_mon > 1 && icaltime_is_leap_year(year))
+        ++days;
+
+    days += tm->tm_mday - 1;
+    hours = days * 24 + tm->tm_hour;
+    minutes = hours * 60 + tm->tm_min;
+    seconds = minutes * 60 + tm->tm_sec;
+
+    return seconds;
+}
+
 struct icaltimetype icaltime_from_timet_with_zone(const time_t tm, const int is_date,
                                                   const icaltimezone *zone)
 {
@@ -215,6 +213,7 @@ struct icaltimetype icaltime_from_timet_with_zone(const time_t tm, const int is_
     utc_zone = icaltimezone_get_utc_timezone();
 
     /* Convert the time_t to a struct tm in UTC time. We can trust gmtime for this. */
+    memset(&t, 0, sizeof(struct tm));
     if (!gmtime_r(&tm, &t))
         return is_date ? icaltime_null_date () : icaltime_null_time ();
 
@@ -243,19 +242,11 @@ struct icaltimetype icaltime_from_timet_with_zone(const time_t tm, const int is_
     return tt;
 }
 
-/**     @brief Convenience constructor.
- *
- * Returns the current time in the given timezone, as an icaltimetype.
- */
 struct icaltimetype icaltime_current_time_with_zone(const icaltimezone *zone)
 {
     return icaltime_from_timet_with_zone(time(NULL), 0, zone);
 }
 
-/**     @brief Convenience constructor.
- *
- * Returns the current day as an icaltimetype, with is_date set.
- */
 struct icaltimetype icaltime_today(void)
 {
     return icaltime_from_timet_with_zone(time(NULL), 1, NULL);
@@ -264,7 +255,7 @@ struct icaltimetype icaltime_today(void)
 time_t icaltime_as_timet(const struct icaltimetype tt)
 {
     struct tm stm;
-    time_t t;
+    time_t t = (time_t)-1;
 
     /* If the time is the special null time, return 0. */
     if (icaltime_is_null_time(tt)) {
@@ -287,19 +278,12 @@ time_t icaltime_as_timet(const struct icaltimetype tt)
     stm.tm_year = tt.year - 1900;
     stm.tm_isdst = -1;
 
-    t = make_time(&stm, 0);
+    if (!make_time(&stm, 0, &t))
+        t = ((time_t)-1);
 
     return t;
 }
 
-/**     Return the time as seconds past the UNIX epoch, using the
- *      given timezone.
- *
- *      This convenience method combines a call to icaltime_convert_to_zone()
- *      with a call to icaltime_as_timet().
- *      If the input timezone is null, no conversion is done; that is, the
- *      time is simply returned as time_t in its native timezone.
- */
 time_t icaltime_as_timet_with_zone(const struct icaltimetype tt, const icaltimezone *zone)
 {
     icaltimezone *utc_zone;
@@ -309,8 +293,8 @@ time_t icaltime_as_timet_with_zone(const struct icaltimetype tt, const icaltimez
 
     utc_zone = icaltimezone_get_utc_timezone();
 
-    /* If the time is the special null time, return 0. */
-    if (icaltime_is_null_time(tt)) {
+    /* Return 0 if the time is the special null time or a bad time */
+    if (icaltime_is_null_time(tt) || !icaltime_is_valid_time(tt)) {
         return 0;
     }
 
@@ -338,10 +322,6 @@ time_t icaltime_as_timet_with_zone(const struct icaltimetype tt, const icaltimez
     return t;
 }
 
-/**
- * Return a string represention of the time, in RFC5545 format. The
- * string is owned by libical.
- */
 const char *icaltime_as_ical_string(const struct icaltimetype tt)
 {
     char *buf;
@@ -351,10 +331,6 @@ const char *icaltime_as_ical_string(const struct icaltimetype tt)
     return buf;
 }
 
-/**
- * Return a string represention of the time, in RFC5545 format. The
- * string is owned by the caller.
- */
 char *icaltime_as_ical_string_r(const struct icaltimetype tt)
 {
     size_t size = 17;
@@ -376,15 +352,7 @@ char *icaltime_as_ical_string_r(const struct icaltimetype tt)
     return buf;
 }
 
-/**
- *      Reset all of the time components to be in their normal ranges. For
- *      instance, given a time with minutes=70, the minutes will be reduces
- *      to 10, and the hour incremented. This allows the caller to do
- *      arithmetic on times without worrying about overflow or
- *      underflow.
- *
- *      Implementation note: we call icaltime_adjust() with no adjustment.
- */
+/* Implementation note: we call icaltime_adjust() with no adjustment. */
 struct icaltimetype icaltime_normalize(const struct icaltimetype tt)
 {
     struct icaltimetype ret = tt;
@@ -393,15 +361,6 @@ struct icaltimetype icaltime_normalize(const struct icaltimetype tt)
     return ret;
 }
 
-/**     @brief Contructor.
- *
- * Create a time from an ISO format string.
- *
- * @todo If the given string specifies a DATE-TIME not in UTC, there
- *       is no way to know if this is a floating time or really refers to a
- *       timezone. We should probably add a new constructor:
- *       icaltime_from_string_with_zone()
- */
 struct icaltimetype icaltime_from_string(const char *str)
 {
     struct icaltimetype tt = icaltime_null_time();
@@ -478,8 +437,6 @@ struct icaltimetype icaltime_from_string(const char *str)
     return icaltime_null_time();
 }
 
-/* Returns whether the specified year is a leap year. Year is the normal year,
-   e.g. 2001. */
 int icaltime_is_leap_year(const int year)
 {
     if (year <= 1752) {
@@ -498,11 +455,11 @@ int icaltime_days_in_year(const int year)
     }
 }
 
-static int _days_in_month[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
+static const int _days_in_month[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
 int icaltime_days_in_month(const int month, const int year)
 {
-    int days = _days_in_month[month];
+    int days;
 
 /* The old code aborting if it was passed a parameter like BYMONTH=0
  * Unfortunately it's not practical right now to pass an error all
@@ -516,6 +473,8 @@ int icaltime_days_in_month(const int month, const int year)
         return 30;
     }
 
+    days = _days_in_month[month];
+
     if (month == 2) {
         days += icaltime_is_leap_year(year);
     }
@@ -542,8 +501,6 @@ int icaltime_day_of_week(const struct icaltimetype t)
     return jt.weekday + 1;
 }
 
-/** Day of the year that the first day of the week (Sunday) is on.
- */
 int icaltime_start_doy_week(const struct icaltimetype t, int fdow)
 {
     UTinstant jt;
@@ -568,10 +525,6 @@ int icaltime_start_doy_week(const struct icaltimetype t, int fdow)
     return jt.day_of_year - delta;
 }
 
-/**
- * @todo Doesn't take into account the start day of the
- * week. strftime assumes that weeks start on Monday.
- */
 int icaltime_week_number(const struct icaltimetype ictt)
 {
     UTinstant jt;
@@ -591,9 +544,6 @@ int icaltime_week_number(const struct icaltimetype ictt)
     return (jt.day_of_year - jt.weekday) / 7;
 }
 
-/**
- *      Returns the day of the year, counting from 1 (Jan 1st).
- */
 int icaltime_day_of_year(const struct icaltimetype t)
 {
     int is_leap = icaltime_is_leap_year(t.year);
@@ -601,11 +551,6 @@ int icaltime_day_of_year(const struct icaltimetype t)
     return days_in_year_passed_month[is_leap][t.month - 1] + t.day;
 }
 
-/**     @brief Contructor.
- *
- *      Create a new time, given a day of year and a year.
- */
-/* Jan 1 is day #1, not 0 */
 struct icaltimetype icaltime_from_day_of_year(const int _doy, const int _year)
 {
     struct icaltimetype tt = icaltime_null_date();
@@ -641,11 +586,6 @@ struct icaltimetype icaltime_from_day_of_year(const int _doy, const int _year)
     return tt;
 }
 
-/**     @brief Constructor.
- *
- *      Return a null time, which indicates no time has been set.
- *      This time represents the beginning of the epoch.
- */
 struct icaltimetype icaltime_null_time(void)
 {
     struct icaltimetype t;
@@ -655,10 +595,6 @@ struct icaltimetype icaltime_null_time(void)
     return t;
 }
 
-/**     @brief Constructor.
- *
- *      Return a null date, which indicates no time has been set.
- */
 struct icaltimetype icaltime_null_date(void)
 {
     struct icaltimetype t;
@@ -678,39 +614,28 @@ struct icaltimetype icaltime_null_date(void)
     return t;
 }
 
-/**
- *      Returns false if the time is clearly invalid, but is not null. This
- *      is usually the result of creating a new time type buy not clearing
- *      it, or setting one of the flags to an illegal value.
- */
 int icaltime_is_valid_time(const struct icaltimetype t)
 {
-    if (t.year < 0 || t.year > 3000 || t.is_date > 1 || t.is_date < 0) {
+    if (t.year < 0 || t.year > 9999 ||
+        t.month < 0 || t.month > 12 ||
+        t.day < 0 || t.day > 31 ||
+        t.is_date > 1 || t.is_date < 0) {
         return 0;
     } else {
         return 1;
     }
 }
 
-/**     @brief Returns true if time is a DATE
- */
 int icaltime_is_date(const struct icaltimetype t)
 {
     return t.is_date;
 }
 
-/**     @brief Returns true if time is relative to UTC zone
- *
- *      @todo  We should only check the zone
- */
 int icaltime_is_utc(const struct icaltimetype t)
 {
     return t.zone == icaltimezone_get_utc_timezone();
 }
 
-/**
- *      Return true if the time is null.
- */
 int icaltime_is_null_time(const struct icaltimetype t)
 {
     if (t.second + t.minute + t.hour + t.day + t.month + t.year == 0) {
@@ -720,12 +645,6 @@ int icaltime_is_null_time(const struct icaltimetype t)
     return 0;
 }
 
-/**
- *      Return -1, 0, or 1 to indicate that a<b, a==b, or a>b.
- *      This calls icaltime_compare function after converting them to the utc
- *      timezone.
- */
-
 int icaltime_compare(const struct icaltimetype a_in, const struct icaltimetype b_in)
 {
     struct icaltimetype a, b;
@@ -781,10 +700,6 @@ int icaltime_compare(const struct icaltimetype a_in, const struct icaltimetype b
     return 0;
 }
 
-/**
- *      like icaltime_compare, but only use the date parts.
- */
-
 int icaltime_compare_date_only(const struct icaltimetype a_in,
                                const struct icaltimetype b_in)
 {
@@ -815,10 +730,6 @@ int icaltime_compare_date_only(const struct icaltimetype a_in,
     return 0;
 }
 
-/**
- *      like icaltime_compare, but only use the date parts; accepts timezone.
- */
-
 int icaltime_compare_date_only_tz(const struct icaltimetype a_in,
                                   const struct icaltimetype b_in,
                                   icaltimezone *tz)
@@ -856,12 +767,6 @@ struct icaldurationtype  icaltime_subtract(struct icaltimetype t1,
                                            struct icaltimetype t2)
 */
 
-/**     @brief Internal, shouldn't be part of the public API
- *
- *      Adds (or subtracts) a time from a icaltimetype.
- *      NOTE: This function is exactly the same as icaltimezone_adjust_change()
- *      except for the type of the first parameter.
- */
 void icaltime_adjust(struct icaltimetype *tt,
                      const int days, const int hours,
                      const int minutes, const int seconds)
@@ -947,16 +852,6 @@ void icaltime_adjust(struct icaltimetype *tt,
     tt->day = day;
 }
 
-/**     @brief Convert time to a given timezone
- *
- *      Convert a time from its native timezone to a given timezone.
- *
- *      If tt is a date, the returned time is an exact
- *      copy of the input. If it's a floating time, the returned object
- *      represents the same time translated to the given timezone.
- *      Otherwise the time will be converted to the new
- *      time zone, and its native timezone set to the right timezone.
- */
 struct icaltimetype icaltime_convert_to_zone(const struct icaltimetype tt, icaltimezone *zone)
 {
     struct icaltimetype ret = tt;
@@ -1000,12 +895,6 @@ const char *icaltime_get_tzid(const struct icaltimetype t)
     }
 }
 
-/**     @brief Set the timezone
- *
- *      Force the icaltime to be interpreted relative to another timezone.
- *      If you need to do timezone conversion, applying offset adjustments,
- *      then you should use icaltime_convert_to_zone instead.
- */
 struct icaltimetype icaltime_set_timezone(struct icaltimetype *t, const icaltimezone *zone)
 {
     /* If it's a date do nothing */
@@ -1022,18 +911,6 @@ struct icaltimetype icaltime_set_timezone(struct icaltimetype *t, const icaltime
     return *t;
 }
 
-/**
- *  @brief builds an icaltimespan given a start time, end time and busy value.
- *
- *  @param dtstart   The beginning time of the span, can be a date-time
- *                   or just a date.
- *  @param dtend     The end time of the span.
- *  @param is_busy   A boolean value, 0/1.
- *  @return          A span using the supplied values.
- *
- *  returned span contains times specified in UTC.
- */
-
 icaltime_span icaltime_span_new(struct icaltimetype dtstart, struct icaltimetype dtend, int is_busy)
 {
     icaltime_span span;
@@ -1044,42 +921,13 @@ icaltime_span icaltime_span_new(struct icaltimetype dtstart, struct icaltimetype
                                              dtstart.zone ? dtstart.
                                              zone : icaltimezone_get_utc_timezone());
 
-    if (icaltime_is_null_time(dtend)) {
-        if (!icaltime_is_date(dtstart)) {
-            /* If dtstart is a DATE-TIME and there is no DTEND nor DURATION
-               it takes no time */
-            span.end = span.start;
-            return span;
-        } else {
-            dtend = dtstart;
-        }
-    }
-
     span.end = icaltime_as_timet_with_zone(dtend,
                                            dtend.zone ? dtend.
                                            zone : icaltimezone_get_utc_timezone());
 
-    if (icaltime_is_date(dtstart)) {
-        /* no time specified, go until the end of the day.. */
-        span.end += 60 * 60 * 24 - 1;
-    }
     return span;
 }
 
-/** @brief Returns true if the two spans overlap
- *
- *  @param s1         1st span to test
- *  @param s2         2nd span to test
- *  @return           boolean value
- *
- *  The result is calculated by testing if the start time of s1 is contained
- *  by the s2 span, or if the end time of s1 is contained by the s2 span.
- *
- *  Also returns true if the spans are equal.
- *
- *  Note, this will return false if the spans are adjacent.
- */
-
 int icaltime_span_overlaps(icaltime_span *s1, icaltime_span *s2)
 {
     /* s1->start in s2 */
@@ -1104,15 +952,6 @@ int icaltime_span_overlaps(icaltime_span *s1, icaltime_span *s2)
     return 0;
 }
 
-/** @brief Returns true if the span is totally within the containing
- *  span
- *
- *  @param s          The span to test for.
- *  @param container  The span to test against.
- *  @return           boolean value.
- *
- */
-
 int icaltime_span_contains(icaltime_span *s, icaltime_span *container)
 {
     if ((s->start >= container->start && s->start < container->end) &&
